@@ -98,20 +98,20 @@ static void glade_util_find_widget_recursive (GtkWidget *widget,
 
 /* This shows a simple dialog box with a label and an 'OK' button.
    Example usage:
-    glade_util_show_message_box ("Error saving file");
-
-    FIXME: NO WAY TO SET THE TRANSIENT PARENT
+    glade_util_show_message_box ("Error saving file", NULL);
 */
 void
-glade_util_show_message_box (const gchar * message)
+glade_util_show_message_box	(const gchar	*message,
+				 GtkWidget	*transient_widget)
 {
+  GtkWidget *toplevel;
+
 #ifdef USE_GNOME
   GtkWidget *dialog, *button;
   dialog = gnome_message_box_new (message, GNOME_MESSAGE_BOX_ERROR,
 				  GNOME_STOCK_BUTTON_OK, NULL);
   button = GNOME_DIALOG (dialog)->buttons->data;
   gtk_widget_grab_focus (button);
-  gtk_widget_show (dialog);
 #else
   GtkWidget *dialog, *label, *button;
 
@@ -140,8 +140,17 @@ glade_util_show_message_box (const gchar * message)
   gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
 		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
 		      GINT_TO_POINTER (GladeEscDestroys));
-  gtk_widget_show (dialog);
 #endif
+
+  if (transient_widget)
+    {
+      toplevel = glade_util_get_toplevel (transient_widget);
+      if (toplevel && GTK_IS_WINDOW (toplevel) && GTK_WIDGET_MAPPED (toplevel))
+	gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (toplevel));
+    }
+
+  gtk_widget_show (dialog);
 }
 
 
@@ -156,18 +165,19 @@ glade_util_show_message_box (const gchar * message)
      gchar *buttons[] = { "Yes", "No", "Cancel" };
      GtkSignalFunc signal_handlers[] = { on_yes, on_no, NULL };
 
-     dialog = glade_util_create_dialog_with_buttons ("Do you want to save the current project?", 3, buttons, 3, signal_handlers, NULL);
+     dialog = glade_util_create_dialog_with_buttons ("Do you want to save the current project?", 3, buttons, 3, signal_handlers, NULL, NULL);
      gtk_widget_show (dialog);
  */
 GtkWidget*
-glade_util_create_dialog_with_buttons (const gchar * message,
-				       gint nbuttons,
-				       const gchar * buttons[],
-				       gint default_button,
-				       GtkSignalFunc signal_handlers[],
-				       gpointer data)
+glade_util_create_dialog_with_buttons	(const gchar	*message,
+					 gint		 nbuttons,
+					 const gchar	*buttons[],
+					 gint		 default_button,
+					 GtkSignalFunc	 signal_handlers[],
+					 gpointer	 data,
+					 GtkWidget	*transient_widget)
 {
-  GtkWidget *dialog, *hbox, *label, *button, *bbox;
+  GtkWidget *dialog, *hbox, *label, *button, *bbox, *toplevel;
   int i;
 
   dialog = gtk_dialog_new ();
@@ -213,6 +223,15 @@ glade_util_create_dialog_with_buttons (const gchar * message,
 			  GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
 			  GINT_TO_POINTER (GladeEscDestroys));
     }
+
+  if (transient_widget)
+    {
+      toplevel = glade_util_get_toplevel (transient_widget);
+      if (toplevel && GTK_IS_WINDOW (toplevel) && GTK_WIDGET_MAPPED (toplevel))
+	gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (toplevel));
+    }
+
   return dialog;
 }
 
@@ -224,23 +243,23 @@ glade_util_create_dialog_with_buttons (const gchar * message,
 
    Example usage:
      glade_util_show_entry_dialog ("Name:", "default", widget, on_dialog_ok,
-				   "NewName");
+				   "NewName", NULL);
 
      void
      on_dialog_ok(GtkWidget *widget, gchar *value, gpointer data)
      {
      ...
 
-     FIXME: NO WAY TO SET THE TRANSIENT PARENT (?)
 */
 void
-glade_util_show_entry_dialog (const gchar * message,
-			      const gchar * initial_value,
-			      GtkWidget * widget,
-			      GbEntryDialogFunc signal_handler,
-			      gpointer data)
+glade_util_show_entry_dialog	(const gchar	*message,
+				 const gchar	*initial_value,
+				 GtkWidget	*widget,
+				 GbEntryDialogFunc signal_handler,
+				 gpointer	 data,
+				 GtkWidget	*transient_widget)
 {
-  GtkWidget *dialog, *hbox, *label, *entry, *button;
+  GtkWidget *dialog, *hbox, *label, *entry, *button, *toplevel;
 
   dialog = gtk_dialog_new ();
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
@@ -292,6 +311,15 @@ glade_util_show_entry_dialog (const gchar * message,
 		      GINT_TO_POINTER (GladeEscDestroys));
   gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 		      GTK_SIGNAL_FUNC (on_entry_dialog_destroy), widget);
+
+  if (transient_widget)
+    {
+      toplevel = glade_util_get_toplevel (transient_widget);
+      if (toplevel && GTK_IS_WINDOW (toplevel) && GTK_WIDGET_MAPPED (toplevel))
+	gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (toplevel));
+    }
+
   gtk_widget_show (dialog);
 }
 
@@ -1220,6 +1248,9 @@ glade_util_get_label_text (GtkWidget *label)
       else
 	in_pattern = FALSE;
       label_with_underscores[j++] = label_wc[i];
+
+      if (label_wc[i] == '_')
+	label_with_underscores[j++] = '_';
     }
   label_with_underscores[j] = '\0';
 
@@ -1352,11 +1383,15 @@ glade_util_copy_file (const gchar *src,
 }
 
 
+/* Creates a directory if it doesn't already exist. directory must be an
+   absolute path. */
 GladeError*
 glade_util_ensure_directory_exists (const gchar *directory)
 {
   struct stat filestat;
 
+  g_return_val_if_fail (g_path_is_absolute (directory), NULL);
+
   if (stat (directory, &filestat) != 0)
     {
       /* If the directory doesn't exist, try to create it. */
@@ -1367,10 +1402,13 @@ glade_util_ensure_directory_exists (const gchar *directory)
 
 	  /* First make sure the parent directory exists. */
 	  parent_dir = glade_util_parent_directory (directory);
-	  error = glade_util_ensure_directory_exists (parent_dir);
-	  g_free (parent_dir);
-	  if (error)
-	    return error;
+	  if (parent_dir)
+	    {
+	      error = glade_util_ensure_directory_exists (parent_dir);
+	      g_free (parent_dir);
+	      if (error)
+		return error;
+	    }
 
 #ifdef _WIN32
 	  if (mkdir (directory) != 0)
@@ -1715,7 +1753,9 @@ glade_util_parent_directory	(const gchar	    *dir)
   gchar *skipped_root, *parent_dir;
   gint pos;
 
-  /* We handle the root dir specially. */
+  g_return_val_if_fail (g_path_is_absolute (dir), NULL);
+
+  /* We handle the root dir specially here, and return NULL. */
   skipped_root = g_path_skip_root ((gchar*) dir);
   if (*skipped_root == '\0')
     return NULL;
@@ -1731,9 +1771,15 @@ glade_util_parent_directory	(const gchar	    *dir)
   if (pos < 0)
     return NULL;
 
+  /* Check if the parent directory is the root directory. If it is, we just
+     want to return the root directory, i.e. "/" or "C:\". */
+  if (pos <= skipped_root - dir)
+    pos = skipped_root - dir;
+
   parent_dir = g_malloc (pos + 1);
   strncpy (parent_dir, dir, pos);
   parent_dir[pos] = '\0';
+
   return parent_dir;
 }
 
@@ -1807,3 +1853,73 @@ glade_util_get_next_free_project_directory (gchar **project_directory_return,
 
   return NULL;
 }
+
+
+/* This will hold the last "TZ=XXX" string we used with putenv(). After we
+   call putenv() again to set a new TZ string, we can free the previous one.
+   As far as I know, no libc implementations actually free the memory used in
+   the environment variables (how could they know if it is a static string or
+   a malloc'ed string?), so we have to free it ourselves. */
+static char* saved_tz = NULL;
+
+
+/* Sets the TZ environment variable to the given value, e.g. "UTC", returning
+   the old setting.
+   NOTE: You must call glade_util_reset_timezone() some time later to restore
+   the original TZ. Pass glade_util_reset_timezone() the string that
+   glade_util_set_timezone() returns. */
+gchar*
+glade_util_set_timezone		(const gchar	*tz)
+{
+  char *old_tz, *old_tz_copy = NULL, *new_tz;
+
+  /* Get the old TZ setting and save a copy of it to return. */
+  old_tz = getenv ("TZ");
+  if (old_tz)
+    {
+      old_tz_copy = g_malloc (strlen (old_tz) + 4);
+      strcpy (old_tz_copy, "TZ=");
+      strcpy (old_tz_copy + 3, old_tz);
+    }
+
+  /* Create the new TZ string. */
+  new_tz = g_malloc (strlen (tz) + 4);
+  strcpy (new_tz, "TZ=");
+  strcpy (new_tz + 3, tz);
+
+  /* Add the new TZ to the environment. */
+  putenv (new_tz); 
+  tzset ();
+
+  /* Free any previous TZ environment string we have used. */
+  if (saved_tz)
+    g_free (saved_tz);
+
+  /* Save a pointer to the TZ string we just set, so we can free it later. */
+  saved_tz = new_tz;
+
+  return old_tz_copy; /* This will be zero if the TZ env var was not set */
+}
+
+
+void
+glade_util_reset_timezone	(gchar		*tz)
+{
+  /* restore the original TZ setting. */
+  if (tz)
+    putenv (tz);
+  else
+    putenv ("TZ"); /* Delete from environment */
+
+  tzset ();
+
+  /* Free any previous TZ environment string we have used. */
+  if (saved_tz)
+    g_free (saved_tz);
+
+  /* Save a pointer to the TZ string we just set, so we can free it later.
+     (This can possibly be NULL if there was no TZ to restore.) */
+  saved_tz = tz;
+}
+
+
